'use strict';

const pragmaDefault = {
  importName: 'default',
  aliasName: 'React',
  moduleName: 'react'
};
const pragmaFragDefault = {
  importName: 'default',
  aliasName: 'React',
  moduleName: 'react'
};

const getPragma = (pragma, pragmaDefault) => {
  // matches: `importName1 from moduleName`
  // matches: `importName1 as aliasName1 from moduleName`
  // matches: `{ importName2 } from moduleName`
  // matches: `{ importName2 as aliasName2 } from moduleName`
  const importMatch = /^\s*(?:([^\s]+)(?:\s+as\s+([^\s]+))?|\{\s*([^\s]+)(?:\s+as\s+([^\s]+))?\s*\})\s+from\s+(.*[^\s])\s*$/;
  const isPragmaMatchable = typeof pragma === 'string' && importMatch.test(pragma);

  if (isPragmaMatchable) {
    const [, importName1, aliasName1, importName2, aliasName2, moduleName] = String(pragma).match(importMatch);
    const hasAlias = Boolean(aliasName1 || aliasName2);
    const isNamedImport = Boolean(aliasName1 || importName2);
    const aliasName = hasAlias ? aliasName1 || aliasName2 : importName1 || importName2;
    const importName = isNamedImport ? importName1 || importName2 : 'default';
    return Object.assign(Object.create(null), pragmaDefault, {
      importName,
      aliasName,
      moduleName
    });
  } else {
    return Object.assign(Object.create(null), pragmaDefault, pragma);
  }
};

const getJsxImportsTransformer = (t, opt) => {
  const transformer = {
    name: 'jsx-imports',
    visitor: {
      JSXElement: {
        enter(path, {
          file
        }) {
          file.set('hasJsxElement', true);
        }

      },
      JSXFragment: {
        enter(path, {
          file
        }) {
          file.set('hasJsxElement', true);
          file.set('hasJsxFragment', true);
        }

      },
      Program: {
        exit(path, {
          file
        }) {
          const importSpecifiersByModuleName = Object.create(null);

          if (file.get('hasJsxElement')) {
            const isAliasAlreadyBound = path.scope.hasBinding(opt.pragma.aliasName);

            if (!isAliasAlreadyBound) {
              importSpecifiersByModuleName[opt.pragma.moduleName] = importSpecifiersByModuleName[opt.pragma.moduleName] || [];
              const isImportNameDefault = opt.pragma.importName === 'default';
              importSpecifiersByModuleName[opt.pragma.moduleName].push(isImportNameDefault ? t.importDefaultSpecifier(t.identifier(opt.pragma.aliasName)) : t.importSpecifier(t.identifier(opt.pragma.aliasName), t.identifier(opt.pragma.importName)));
            }
          }

          if (file.get('hasJsxFragment')) {
            const isAliasAlreadyBound = path.scope.hasBinding(opt.pragmaFrag.aliasName);

            if (!isAliasAlreadyBound) {
              importSpecifiersByModuleName[opt.pragmaFrag.moduleName] = importSpecifiersByModuleName[opt.pragmaFrag.moduleName] || [];
              const isImportNameDefault = opt.pragmaFrag.importName === 'default';
              importSpecifiersByModuleName[opt.pragmaFrag.moduleName].push(isImportNameDefault ? t.importDefaultSpecifier(t.identifier(opt.pragmaFrag.aliasName)) : t.importSpecifier(t.identifier(opt.pragmaFrag.aliasName), t.identifier(opt.pragmaFrag.importName)));
            }
          }

          Object.keys(importSpecifiersByModuleName).forEach(moduleName => {
            let hasDefaultSpecifier = false;
            const specifiers = importSpecifiersByModuleName[moduleName].filter(specifier => {
              const isDefaultSpecifier = specifier.type === 'ImportDefaultSpecifier';

              if (hasDefaultSpecifier && isDefaultSpecifier) {
                return false;
              }

              hasDefaultSpecifier = hasDefaultSpecifier || isDefaultSpecifier;
              return true;
            });
            path.unshiftContainer('body', t.importDeclaration(specifiers, t.stringLiteral(moduleName)));
          });
        }

      }
    }
  };
  return transformer;
};

const initializer = (api, opt) => {
  opt = Object.assign(Object.create(null), opt);
  opt.pragma = getPragma(opt.pragma, pragmaDefault);
  opt.pragmaFrag = getPragma(opt.pragmaFrag, pragmaFragDefault);
  const transformer = getJsxImportsTransformer(api.types, opt);
  return transformer;
};

const initializerClone = initializer.bind(null);
const pluginExtensions = {
  getPragma,
  pragmaDefault,
  pragmaFragDefault
};
const plugin = Object.assign(initializerClone, pluginExtensions);

module.exports = plugin;
//# sourceMappingURL=index.js.map
